#include "validation.h"

#include <algorithm>
#include <cstdint>
#include <deque>
#include <string>
#include <vector>

// This program will be called as
// output_validator input answer < team_output
//
// Please check the grammar of the team output using the Validator class.
// See input_validator.cpp for information on how to use the Validator class.
// You should also check the validity of the answer here.
// For example, check that a tree printed by the team is a tree indeed.

int main(int argc, char** argv) {
	// The testcase .in file.
	std::ifstream in(argv[1]);
	// The testcase .ans file, if needed for custom validation.
	std::ifstream ans(argv[2]);
	OutputValidator v(argc, argv);

	int input_n, input_t;
	in >> input_n >> input_t;

	// Use an order statistic tree here in case we want to have higher values of T.
	std::vector<std::string> lru;
	for(int i = 0; i < input_n; i++) {
		lru.push_back(std::string("") + (char)(i + 'a'));
	}

	int total = v.read_integer("total", 1, 10000);
	v.check(total == (input_t + input_n - 1) / input_n, "Number of items ", total,
	        " is not minimal");
	v.newline();

	std::vector<std::string> const items = v.read_strings("answer", total, 1, 1);

	int64_t cost = 0;
	for(auto const& item : items) {
		auto const it = std::find(lru.begin(), lru.end(), item);
		v.check(it != lru.end(), "Answer item ", item, " is not present in the list");
		cost += (it - lru.begin()) + 1;
		std::rotate(lru.begin(), it, it + 1);
	}

	v.check(cost == input_t, "The answer cost does not match the input! Found ", cost,
	        " but wanted ", input_t);
}
