#include "validation.h"

#include <signal.h>
#include <thread>

int n = 0;
std::vector<int> a;
bool done = false;

void signal_handler(int x) {
	done = true;
}

void print_vector() {
	std::cout << n << std::endl;
	assert(a.size() == n);

	for(int i = 0; i < a.size() and std::cout.good() and not done; ++i) {
		std::cout << a[i] << " \n"[i == n - 1];
	}
	std::cout.flush();
}

int main(int argc, char** argv) {
	signal(SIGPIPE, &signal_handler);
	// All 3 are needed to prevent blocking on the output thread.
	std::cin.tie(nullptr);
	std::cout.tie(nullptr);
	std::cerr.tie(nullptr);
	std::iostream::sync_with_stdio(false);

	std::ifstream in(argv[1]);

	int maxa;
	unsigned int testcase_seed, fixed_seed;
	in >> n >> maxa >> testcase_seed;

	// Generate the team input.
	std::random_device dev;
	auto random_seed = dev();

	auto seed = testcase_seed ^ random_seed;

	if(in >> fixed_seed) {
		seed = fixed_seed;
	}

	std::cerr << "Seed: " << seed << std::endl;

	std::mt19937 rng(seed);
	a.resize(n);
	std::uniform_int_distribution<int> dist(1, maxa);
	for(auto& x : a) {
		x = dist(rng);
	}

	// Print the vector and in parallel read the team answer.
	// This is needed in case the team prints an answer before reading all input, and the pipes fill
	// up.
	std::thread t(print_vector);

	// Validate the team output.
	OutputValidator v(argc, argv);
	int i = v.read_integer("i", 1, n);
	v.space();
	int j = v.read_integer("j", 1, n);
	v.newline();
	v.check(i != j, "Team printed the same index twice!");

	auto x = a[i - 1];
	auto y = a[j - 1];
	v.check(y % x == 0, "Value ", x, " at position ", i, " does not divide value ", y,
	        " at position ", j);
}
